@@ -31,6 +31,70 @@ pub fn patch<DSP, MSG>(
     focused_node: &mut Option<Node>,
     patches: Vec<Patch<MSG>>,
 ) -> Result<ActiveClosure, JsValue>
+where
+    MSG: 'static,
+    DSP: Clone + Dispatch<MSG> + 'static,
+{
+    patch_by_traversal_path(
+        program,
+        root_node,
+        old_closures,
+        focused_node,
+        patches,
+    )
+}
+
+/// patch using the tree path traversal instead of node_idx
+pub fn patch_by_traversal_path<DSP, MSG>(
+    program: Option<&DSP>,
+    root_node: &mut Node,
+    old_closures: &mut ActiveClosure,
+    focused_node: &mut Option<Node>,
+    patches: Vec<Patch<MSG>>,
+) -> Result<ActiveClosure, JsValue>
+where
+    MSG: 'static,
+    DSP: Clone + Dispatch<MSG> + 'static,
+{
+    let nodes_to_find: Vec<(&[usize], Option<&&'static str>)> = patches
+        .iter()
+        .map(|patch| (patch.path(), patch.tag()))
+        .collect();
+    let mut active_closures = HashMap::new();
+    let nodes_to_patch =
+        find_all_nodes_by_path(root_node.clone(), &nodes_to_find);
+    for patch in patches.iter() {
+        let patch_path = patch.path();
+        if let Some(element) = nodes_to_patch.get(patch_path) {
+            let new_closures = apply_patch_to_node(
+                program,
+                root_node,
+                &element,
+                old_closures,
+                focused_node,
+                &patch,
+            )?;
+            active_closures.extend(new_closures);
+        } else {
+            unreachable!("Getting here means we didn't find the element of next node that we are supposed to patch, patch_path: {:?} node_idx: {}", patch_path, patch.node_idx());
+        }
+    }
+    Ok(active_closures)
+}
+
+/// Apply all of the patches to our old root node in order to create the new root node
+/// that we desire.
+/// This is usually used after diffing two virtual nodes.
+///
+/// Note: If Program is None, it is a dumb patch, meaning
+/// there is no event listener attached or changed
+pub fn patch_by_node_idx_traversal<DSP, MSG>(
+    program: Option<&DSP>,
+    root_node: &mut Node,
+    old_closures: &mut ActiveClosure,
+    focused_node: &mut Option<Node>,
+    patches: Vec<Patch<MSG>>,
+) -> Result<ActiveClosure, JsValue>
 where
     MSG: 'static,
     DSP: Clone + Dispatch<MSG> + 'static,
@@ -43,7 +107,7 @@ where
 
     // finding the nodes to be patched before hand, instead of calling it
     // in every patch loop.
-    let nodes_to_patch = find_nodes(root_node, &patches);
+    let nodes_to_patch = find_nodes_by_node_idx(root_node, &patches);
 
     #[cfg(feature = "with-measure")]
     let t2 = {
@@ -103,13 +167,14 @@ where
 /// elements are it's descendant before it could reach the elements in the bottom.
 ///
 /// Complexity: O(n), where n is the total number of html nodes
-fn find_nodes<MSG>(
+fn find_nodes_by_node_idx<MSG>(
     root_node: &Node,
     patches: &[Patch<MSG>],
 ) -> HashMap<usize, Node> {
-    let mut nodes_to_find = HashMap::new();
+    let mut nodes_to_find: HashMap<usize, Option<&&'static str>> =
+        HashMap::new();
 
-    let mut nodes_to_patch = HashMap::new();
+    let mut nodes_to_patch: HashMap<usize, Node> = HashMap::new();
 
     for patch in patches {
         nodes_to_find.insert(patch.node_idx(), patch.tag());
@@ -118,19 +183,41 @@ fn find_nodes<MSG>(
     #[cfg(feature = "with-measure")]
     log::trace!("there are {} nodes_to_find", nodes_to_find.len());
 
-    find_nodes_recursive(
+    find_nodes_by_idx_recursive(
         root_node,
         &mut 0,
         &nodes_to_find,
         &mut nodes_to_patch,
     );
+
+    let not_found = list_nodes_not_found(&nodes_to_find, &nodes_to_patch);
+    if !not_found.is_empty() {
+        log::warn!("These are not found: {:#?}", not_found);
+    }
+
     nodes_to_patch
 }
 
+fn list_nodes_not_found(
+    nodes_to_find: &HashMap<usize, Option<&&'static str>>,
+    nodes_to_patch: &HashMap<usize, Node>,
+) -> Vec<usize> {
+    let mut not_found = vec![];
+    for (idx, tag) in nodes_to_find.iter() {
+        if nodes_to_patch.contains_key(idx) {
+            // found
+        } else {
+            log::warn!("not found.. {} - {:?}", idx, tag);
+            not_found.push(*idx);
+        }
+    }
+    not_found
+}
+
 /// find the html nodes recursively
 /// early returns true if all node has been found
 /// before completing iterating all the elements
-fn find_nodes_recursive(
+fn find_nodes_by_idx_recursive(
     node: &Node,
     cur_node_idx: &mut usize,
     nodes_to_find: &HashMap<usize, Option<&&'static str>>,
@@ -155,7 +242,7 @@ fn find_nodes_recursive(
     for i in 0..child_node_count {
         let child_node = children.item(i).expect("Expecting a child node");
         *cur_node_idx += 1;
-        if find_nodes_recursive(
+        if find_nodes_by_idx_recursive(
             &child_node,
             cur_node_idx,
             nodes_to_find,
@@ -167,6 +254,45 @@ fn find_nodes_recursive(
     false
 }
 
+fn find_node_by_path_recursive(
+    node: Node,
+    path: &mut Vec<usize>,
+) -> Option<Node> {
+    if path.is_empty() {
+        Some(node)
+    } else {
+        let idx = path.remove(0);
+        let children = node.child_nodes();
+        if let Some(child) = children.item(idx as u32) {
+            find_node_by_path_recursive(child, path)
+        } else {
+            None
+        }
+    }
+}
+
+fn find_all_nodes_by_path(
+    node: Node,
+    nodes_to_find: &Vec<(&[usize], Option<&&'static str>)>,
+) -> HashMap<Vec<usize>, Node> {
+    let mut nodes_to_patch: HashMap<Vec<usize>, Node> =
+        HashMap::with_capacity(nodes_to_find.len());
+
+    for (path, tag) in nodes_to_find {
+        let mut traverse_path = path.to_vec();
+        let root_idx = traverse_path.remove(0);
+        assert_eq!(0, root_idx, "path should start at 0");
+        if let Some(found) =
+            find_node_by_path_recursive(node.clone(), &mut traverse_path)
+        {
+            nodes_to_patch.insert(path.to_vec(), found);
+        } else {
+            log::warn!("can not find: {:?} {:?}", path, tag);
+        }
+    }
+    nodes_to_patch
+}
+
 /// Get the "data-sauron-vdom-id" of all the desendent of this node including itself
 /// This is needed to free-up the closure that was attached ActiveClosure manually
 /// TODO: Make a test when an element is removed, all of it's descendant closure should also be
@@ -282,8 +408,7 @@ where
     match patch {
         Patch::InsertNode(InsertNode {
             tag: _,
-            node_idx: _,
-            new_node_idx,
+            patch_path: _,
             node: for_insert,
         }) => {
             let element: &Element = node.unchecked_ref();
@@ -291,7 +416,6 @@ where
                 program,
                 &for_insert,
                 focused_node,
-                &mut Some(*new_node_idx),
             );
             let parent_node =
                 element.parent_node().expect("must have a parent node");
@@ -342,8 +466,7 @@ where
         //
         Patch::ReplaceNode(ReplaceNode {
             tag,
-            node_idx,
-            new_node_idx,
+            patch_path,
             replacement,
         }) => {
             let element: &Element = node.unchecked_ref();
@@ -351,9 +474,8 @@ where
                 program,
                 replacement,
                 focused_node,
-                &mut Some(*new_node_idx),
             );
-            if *node_idx == 0 {
+            if patch_path.node_idx() == 0 {
                 *root_node = created_node.node.clone();
             }
             if let Some(tag) = tag {
@@ -394,17 +516,16 @@ where
         }
         Patch::AppendChildren(AppendChildren {
             tag: _,
-            node_idx: _,
+            patch_path: _,
             children: new_nodes,
         }) => {
             let element: &Element = node.unchecked_ref();
             let mut active_closures = HashMap::new();
-            for (append_children_node_idx, new_node) in new_nodes.iter() {
+            for (_append_children_node_idx, new_node) in new_nodes.iter() {
                 let created_node = CreatedNode::create_dom_node_opt::<DSP, MSG>(
                     program,
                     &new_node,
                     focused_node,
-                    &mut Some(*append_children_node_idx),
                 );
                 element.append_child(&created_node.node)?;
                 active_closures.extend(created_node.closures);
