@@ -1,9 +1,6 @@
 #![deny(warnings)]
 use sauron_core::{
-    html::{
-        attributes::*,
-        *,
-    },
+    html::{attributes::*, *},
     mt_dom::patch::*,
     *,
 };
@@ -122,14 +119,48 @@ fn subsequent_updates() {
     assert_eq!(
         patches1,
         vec![
-            ChangeText::new(4, &Text::new("0"), 9, &Text::new("1")).into(),
-            ChangeText::new(9, &Text::new("1"), 14, &Text::new("2")).into(),
-            ChangeText::new(14, &Text::new("2"), 19, &Text::new("3")).into(),
-            ChangeText::new(19, &Text::new("3"), 24, &Text::new("4")).into(),
+            ChangeText::new(
+                &Text::new("0"),
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 0, 0, 0, 0,]),
+                    TreePath::start_at(9, vec![0, 0, 0, 0, 0,])
+                ),
+                &Text::new("1")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("1"),
+                PatchPath::new(
+                    TreePath::start_at(9, vec![0, 0, 1, 0, 0,]),
+                    TreePath::start_at(14, vec![0, 0, 1, 0, 0,])
+                ),
+                &Text::new("2")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("2"),
+                PatchPath::new(
+                    TreePath::start_at(14, vec![0, 0, 2, 0, 0,]),
+                    TreePath::start_at(19, vec![0, 0, 2, 0, 0,])
+                ),
+                &Text::new("3")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("3"),
+                PatchPath::new(
+                    TreePath::start_at(19, vec![0, 0, 3, 0, 0,]),
+                    TreePath::start_at(24, vec![0, 0, 3, 0, 0,])
+                ),
+                &Text::new("4")
+            )
+            .into(),
             InsertNode::new(
                 Some(&"section"),
-                2,
-                2,
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0]),
+                    TreePath::start_at(2, vec![0, 0, 0])
+                ),
                 &div(
                     vec![key("hashXXX")],
                     vec![
@@ -300,19 +331,61 @@ fn subsequent_updates() {
     );
 
     let patches2 = diff(&update1, &update2);
-    log::trace!("patches2: {:#?}", patches2);
+    log::trace!("-->patches2: {:#?}", patches2);
     assert_eq!(
         patches2,
         vec![
-            ChangeText::new(4, &Text::new("0"), 9, &Text::new("1")).into(),
-            ChangeText::new(9, &Text::new("1"), 14, &Text::new("2")).into(),
-            ChangeText::new(14, &Text::new("2"), 19, &Text::new("3")).into(),
-            ChangeText::new(19, &Text::new("3"), 24, &Text::new("4")).into(),
-            ChangeText::new(24, &Text::new("4"), 29, &Text::new("5")).into(),
+            ChangeText::new(
+                &Text::new("0"),
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 0, 0, 0, 0,]),
+                    TreePath::start_at(9, vec![0, 0, 0, 0, 0,])
+                ),
+                &Text::new("1")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("1"),
+                PatchPath::new(
+                    TreePath::start_at(9, vec![0, 0, 1, 0, 0,]),
+                    TreePath::start_at(14, vec![0, 0, 1, 0, 0,])
+                ),
+                &Text::new("2")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("2"),
+                PatchPath::new(
+                    TreePath::start_at(14, vec![0, 0, 2, 0, 0,]),
+                    TreePath::start_at(19, vec![0, 0, 2, 0, 0,])
+                ),
+                &Text::new("3")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("3"),
+                PatchPath::new(
+                    TreePath::start_at(19, vec![0, 0, 3, 0, 0,]),
+                    TreePath::start_at(24, vec![0, 0, 3, 0, 0,])
+                ),
+                &Text::new("4")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("4"),
+                PatchPath::new(
+                    TreePath::start_at(24, vec![0, 0, 4, 0, 0,]),
+                    TreePath::start_at(29, vec![0, 0, 4, 0, 0,])
+                ),
+                &Text::new("5")
+            )
+            .into(),
             InsertNode::new(
                 Some(&"section"),
-                2,
-                2,
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0,]),
+                    TreePath::start_at(2, vec![0, 0, 0,])
+                ),
                 &div(
                     vec![key("hashYYY")],
                     vec![
@@ -427,20 +500,70 @@ fn subsequent_updates() {
     );
 
     let patches3 = diff(&update2, &update3);
-    log::trace!("patches2: {:#?}", patches3);
+    log::trace!("\n---->patches3: {:#?}", patches3);
     assert_eq!(
         patches3,
         vec![
-            ChangeText::new(4, &Text::new("0"), 9, &Text::new("1")).into(),
-            ChangeText::new(9, &Text::new("1"), 14, &Text::new("2")).into(),
-            ChangeText::new(14, &Text::new("2"), 19, &Text::new("3")).into(),
-            ChangeText::new(19, &Text::new("3"), 24, &Text::new("4")).into(),
-            ChangeText::new(24, &Text::new("4"), 29, &Text::new("5")).into(),
-            ChangeText::new(29, &Text::new("5"), 34, &Text::new("6")).into(),
+            ChangeText::new(
+                &Text::new("0"),
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 0, 0, 0, 0,]),
+                    TreePath::start_at(9, vec![0, 0, 0, 0, 0,])
+                ),
+                &Text::new("1")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("1"),
+                PatchPath::new(
+                    TreePath::start_at(9, vec![0, 0, 1, 0, 0,]),
+                    TreePath::start_at(14, vec![0, 0, 1, 0, 0,])
+                ),
+                &Text::new("2")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("2"),
+                PatchPath::new(
+                    TreePath::start_at(14, vec![0, 0, 2, 0, 0,]),
+                    TreePath::start_at(19, vec![0, 0, 2, 0, 0,])
+                ),
+                &Text::new("3")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("3"),
+                PatchPath::new(
+                    TreePath::start_at(19, vec![0, 0, 3, 0, 0,]),
+                    TreePath::start_at(24, vec![0, 0, 3, 0, 0,])
+                ),
+                &Text::new("4")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("4"),
+                PatchPath::new(
+                    TreePath::start_at(24, vec![0, 0, 4, 0, 0,]),
+                    TreePath::start_at(29, vec![0, 0, 4, 0, 0,])
+                ),
+                &Text::new("5")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("5"),
+                PatchPath::new(
+                    TreePath::start_at(29, vec![0, 0, 5, 0, 0,]),
+                    TreePath::start_at(34, vec![0, 0, 5, 0, 0,])
+                ),
+                &Text::new("6")
+            )
+            .into(),
             InsertNode::new(
                 Some(&"section"),
-                2,
-                2,
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0,]),
+                    TreePath::start_at(2, vec![0, 0, 0,])
+                ),
                 &div(
                     vec![key("hashZZZ")],
                     vec![
